> module Nat where

> data Nat = Zero | Succ !Nat
>            deriving (Eq, Ord)

> fromNat :: Num a => Nat -> a
> fromNat Zero     = 0
> fromNat (Succ n) = 1 + fromNat n

> toNat :: Integral a => a -> Nat
> toNat i | i <  0 = error "Negative natural number"
>         | i == 0 = Zero
>         | i >  0 = Succ (toNat (pred i)) 

> instance Enum Nat where
>   succ          = Succ
>
>   pred (Succ n) = n
>   pred Zero     = error "Predeccessor of Zero"
>
>   toEnum        = toNat 
>
>   fromEnum      = fromNat


> instance Show Nat where
>   show n = show (fromNat n :: Integer)

> instance Num Nat where
>   Zero     + n    =  n
>   (Succ m) + n    =  Succ (m + n)
>
>   Zero     * n    =  Zero
>   (Succ m) * n    =  n + m * n
>
>   signum Zero     =  0
>   signum (Succ m) =  1
>
>   fromInteger     =  toNat
>
>   abs             =  id
>
>   negate n        =  error "Cannot negate natural numbers"
>
>   n - Zero            =  n
>   (Succ m) - (Succ n) =  m - n
>   Zero - (Succ _)     =  error "Negative natural number" 

> natrec :: a -> (a -> a) -> Nat -> a
> natrec a f Zero     = a
> natrec a f (Succ n) = natrec (f a) f n
