> module Shapes where

> import Utilities
> import Graphics.HGL

> type Radius    = Int
> type Dimension = Int

---------------------------------------------------------------------------------------

-- Utitlity Functions --

> deg2rad                         :: Angle -> Angle
> deg2rad x                       =  (x*pi) / 180

> allRads                         :: Angle
> allRads                         =  deg2rad 360

pointOnArc measures from 3 O'Clock in a clockwise direction around the circle.

> pointOnArc                      :: Point -> Radius -> Angle -> Point
> pointOnArc (x,y) r a            =  (x + truncate (r' * cos a'), y + truncate (r' * sin a'))
>                                    where r' = fromIntegral r
>                                          a' = deg2rad a

> rotatePoint                     :: Angle -> Point -> Point
> rotatePoint a (x,y)             =  (truncate x'', truncate y'')
>                                    where a'  = deg2rad a
>                                          x'  = fromIntegral x
>                                          y'  = fromIntegral y
>                                          x'' = x' * cos a' - y' * sin a'
>                                          y'' = x' * sin a' + y' * cos a'

> appendLast                      :: [a] -> [a]
> appendLast []                   =  []
> appendLast as                   =  (last as):as

> pplus                           :: Point -> Point -> Point
> (x,y) `pplus` (x2,y2)           =  (x+x2, y+y2)

> pminus                          :: Point -> Point -> Point
> (x,y) `pminus` (x2,y2)          =  (x-x2, y-y2)


----------------------------------------------------------------------------

> full                            :: [Point] -> Graphic
> full                            =  polygon

> empty                           :: [Point] -> Graphic
> empty                           =  polyline . appendLast


> rectangle                       :: Point -> Size -> [Point]
> rectangle (x1,y1) (x,y)         =  [(x1,y1),(x1,y2),(x2, y2),(x2,y1)]
>                                    where
>                                      x2 = x1 + x
>                                      y2 = y1 + y

> emptyRectangle                  :: Point -> Size -> Graphic
> emptyRectangle p                =  empty . rectangle p

> fullRectangle                   :: Point -> Size -> Graphic
> fullRectangle p                 =  full . rectangle p


> emptySquare                     :: Point -> Dimension -> Graphic
> emptySquare p d                 =  emptyRectangle p (d,d)

> fullSquare                      :: Point -> Dimension -> Graphic
> fullSquare p d                  =  fullRectangle p (d,d)

> fullCircle                      :: Point -> Radius -> Graphic
> fullCircle (x,y) r              =  ellipse (x-r,y-r) (x+r,y+r)

> polycos                         :: Int -> Point -> Radius -> [Point]
> polycos n p r                   =  [ pointOnArc p r a | a <- map (*(360.0 / n')) [0.0..(n'-1.0)]]
>                                    where n' = fromIntegral n

> emptyNagon                      :: Int -> Point -> Radius -> Graphic
> emptyNagon n p                  =  empty . polycos n p

> fullNagon                       :: Int -> Point -> Radius -> Graphic
> fullNagon n p                   =  full . polycos n p

> emptyCircle                     :: Point -> Radius -> Graphic
> emptyCircle                     =  emptyNagon 360

> fivePointStar                   :: Point -> Radius -> Graphic
> fivePointStar p r               =  polyline [ pointOnArc p r a | a <- map (*72) [0,2,4,1,3,0]]

> pentagram                       :: Point -> Radius -> Graphic
> pentagram p r                   =  overGraphic (emptyCircle p r) (fivePointStar p r)

--------------------------------------------------------------------------------------------------------

> polycosA                        :: Int -> Point -> Radius -> Angle -> [Point]
> polycosA n p r a                =  [ pointOnArc p r a' | a' <- rs' ]
>                                    where n'  = fromIntegral n
>                                          ns  = [0.0..(n'-1.0)]
>                                          rs  = map (*(360.0 / n')) ns
>                                          rs' = map (+a) rs

> fullNagonA                      :: Int -> Point -> Radius -> Angle -> Graphic
> fullNagonA n p r a              =  full $ polycosA n p r a

> emptyNagonA                     :: Int -> Point -> Radius -> Angle -> Graphic
> emptyNagonA n p r a             =  empty $ polycosA n p r a

> rectangleA                      :: Point -> Size -> Angle -> [Point]
> rectangleA p s a                =  map (pplus p . rotatePoint a) (rectangle (0,0) s)

> emptyRectangleA                 :: Point -> Size -> Angle -> Graphic
> emptyRectangleA p s             =  empty . rectangleA p s

> fullRectangleA                  :: Point -> Size -> Angle -> Graphic
> fullRectangleA p s              =  full . rectangleA p s

> emptySquareA                    :: Point -> Dimension -> Angle -> Graphic
> emptySquareA p d                =  emptyRectangleA p (d,d)

> fullSquareA                     :: Point -> Dimension -> Angle -> Graphic
> fullSquareA p d                 =  fullRectangleA p (d,d)
